@@ -68,6 +68,7 @@ clearlooks_set_widget_parameters (const GtkWidget      *widget,
 	params->corners    = CL_CORNER_ALL;
 		
 	params->focus      = widget && GTK_WIDGET_HAS_FOCUS (widget);
+	params->is_default = widget && GTK_WIDGET_HAS_DEFAULT (widget);
 		
 	if (!params->active && widget && GTK_IS_TOGGLE_BUTTON (widget))
 		params->active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
@@ -99,7 +100,7 @@ draw_shadow (DRAW_ARGS)
 
 	sanitize_size (window, &width, &height);
 
-	if (DETAIL ("entry"))
+	if (DETAIL ("entry") && !(widget && widget->parent && GTK_IS_TREE_VIEW (widget->parent)))
 	{
 		WidgetParameters params;
 		
@@ -201,7 +202,8 @@ draw_box_gap (DRAW_ARGS,
 	}
 	else
 	{
-		printf("box_gap: %s %s\n", detail, G_OBJECT_TYPE_NAME (widget));
+		if (widget)
+			printf("box_gap: %s %s\n", detail, G_OBJECT_TYPE_NAME (widget));
 		parent_class->draw_box_gap (style, window, state_type, shadow_type,
 									   area, widget, detail,
 									   x, y, width, height,
@@ -255,11 +257,15 @@ draw_handle (DRAW_ARGS, GtkOrientation orientation)
 	ClearlooksStyle  *clearlooks_style = CLEARLOOKS_STYLE (style);
 	ClearlooksColors *colors = &clearlooks_style->colors;
 	cairo_t          *cr;
+	gboolean         is_horizontal;
 	
 	sanitize_size (window, &width, &height);
 	
 	cr = clearlooks_begin_paint (window, area);
 	
+	// Evil hack to work around broken orientation for toolbars
+	is_horizontal = (width > height);
+	
 	if (DETAIL ("handlebox"))
 	{
 		WidgetParameters params;
@@ -267,8 +273,9 @@ draw_handle (DRAW_ARGS, GtkOrientation orientation)
 
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
 		handle.type = CL_HANDLE_TOOLBAR;
-		handle.horizontal = FALSE;
+		handle.horizontal = is_horizontal;
 		
+		// Is this ever true? -Daniel
 		if (GTK_IS_TOOLBAR (widget) && shadow_type != GTK_SHADOW_NONE)
 		{
 			cairo_save (cr);
@@ -299,8 +306,9 @@ draw_handle (DRAW_ARGS, GtkOrientation orientation)
 
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
 		handle.type = CL_HANDLE_TOOLBAR;
-		handle.horizontal = FALSE;
+		handle.horizontal = is_horizontal;
 		
+		// Is this ever true? -Daniel
 		if (GTK_IS_TOOLBAR (widget) && shadow_type != GTK_SHADOW_NONE)
 		{
 			cairo_save (cr);
@@ -365,20 +373,17 @@ draw_box (DRAW_ARGS)
 	else if (height == -1)
 		gdk_window_get_size (window, NULL, &height);
 
-	if (DETAIL ("menubar"))
+	if (DETAIL ("menubar") && 
+	    !(widget && (cl_is_panel_widget (widget->parent))))
 	{
 		WidgetParameters params;
 		
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
 
-//		if (gtk_widget_get_toplevel (widget)->allocation.x == widget->allocation.x &&
-//			gtk_widget_get_toplevel (widget)->allocation.width == widget->allocation.width)
-//		{
-			clearlooks_draw_menubar (cr, colors, &params,
-		    	                     x, y, width, height);
-//		}
+		clearlooks_draw_menubar (cr, colors, &params,
+		                         x, y, width, height);
 	}
-	else if (DETAIL ("button") && widget->parent &&
+	else if (DETAIL ("button") && widget && widget->parent &&
                  (GTK_IS_TREE_VIEW(widget->parent) ||
                   GTK_IS_CLIST (widget->parent)))
 	{
@@ -443,10 +448,7 @@ draw_box (DRAW_ARGS)
 		}
 		else
 			params.corners    = CL_CORNER_ALL;		
-		
-		if (DETAIL ("spinbutton"))
-			params.disabled = GTK_WIDGET_STATE (widget) & GTK_STATE_INSENSITIVE;
-		
+	
 		if (GTK_IS_TOGGLE_BUTTON (widget) &&
 		    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
 			params.active = TRUE;
@@ -589,6 +591,8 @@ draw_box (DRAW_ARGS)
 				cl_progressbar_add ((gpointer)widget);
 			}
 		}
+
+		frame = cl_async_animation_getdata((gpointer)widget).frame;
 #endif
 		
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
@@ -600,10 +604,6 @@ draw_box (DRAW_ARGS)
 		
 		cairo_reset_clip (cr);
 		
-#ifdef HAVE_ANIMATION
-		frame = cl_async_animation_getdata((gpointer)widget).frame;
-#endif
-		
 		clearlooks_draw_progressbar_fill (cr, colors, &params, &progressbar,
 		                                  x, y, width, height, frame);
 	}
@@ -647,24 +647,25 @@ draw_box (DRAW_ARGS)
 		WidgetParameters params;
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
 		
-	/*	if (widget && GTK_IS_MENU_BAR (widget->parent))
+		if (widget && GTK_IS_MENU_BAR (widget->parent))
 		{
-			params.corners = CL_CORNER_NONE;
+			/*params.corners = CL_CORNER_NONE;
 			params.active = TRUE;
 			params.state_type = CL_STATE_ACTIVE;
 			params.xthickness = 2;
 			params.ythickness = 2;
 			
-			clearlooks_draw_button (cr, colors, &params, x, y, width, height+1);
+			clearlooks_draw_button (cr, colors, &params, x, y, width, height+1);*/
+			
+			params.corners = CL_CORNER_TOPLEFT | CL_CORNER_TOPRIGHT;
+			height += 1;
 		}
-		else*/
+		else
 		{	
-			WidgetParameters params;
-			clearlooks_set_widget_parameters (widget, style, state_type, &params);
-		
-			clearlooks_draw_menuitem (cr, colors, &params,
-			                          x, y, width, height);
+			params.corners = CL_CORNER_ALL;
 		}
+		
+		clearlooks_draw_menuitem (cr, colors, &params, x, y, width, height);
 	}
 	else if (DETAIL ("hscrollbar") || DETAIL ("vscrollbar") || DETAIL ("slider") || DETAIL ("stepper"))
 	{
@@ -706,7 +707,8 @@ draw_box (DRAW_ARGS)
 	}
 	else if (DETAIL ("toolbar") || DETAIL ("handlebox_bin") || DETAIL ("dockitem_bin"))
 	{
-		if (shadow_type != GTK_SHADOW_NONE)
+		// Only draw the shadows on horizontal toolbars
+		if (shadow_type != GTK_SHADOW_NONE && height < 2*width )
 			clearlooks_draw_toolbar (cr, colors, NULL, x, y, width, height);
 	}
 	else if (DETAIL ("trough"))
@@ -1057,6 +1059,29 @@ draw_resize_grip (GtkStyle       *style,
                   gint            width,
                   gint            height)
 {
+	ClearlooksStyle *clearlooks_style = CLEARLOOKS_STYLE (style);
+	ClearlooksColors *colors = &clearlooks_style->colors;
+
+	cairo_t *cr = clearlooks_begin_paint (window, area);
+
+	WidgetParameters params;
+	ResizeGripParameters grip;
+	grip.edge = (ClearlooksWindowEdge)edge;
+
+	int lx, ly;
+
+	g_return_if_fail (window != NULL);
+
+	if (edge != GDK_WINDOW_EDGE_SOUTH_EAST)
+		return; // sorry... need to work on this :P
+
+	clearlooks_set_widget_parameters (widget, style, state_type, &params);	
+
+	clearlooks_draw_resize_grip (cr, colors, &params, &grip,
+	                             x, y, width, height);
+
+	cairo_destroy (cr);
+
 	return;
 }
 
@@ -1094,7 +1119,7 @@ draw_arrow (GtkStyle  *style,
 		cairo_set_source_rgb (cr, 1, 0, 0);
 		cairo_fill (cr);
 	*/	
-		if (is_combo_box (widget))
+		if (cl_is_combo_box (widget))
 		{
 			arrow.type = CL_ARROW_COMBO;
 			y -= 2;
@@ -1122,7 +1147,7 @@ clearlooks_style_init_from_rc (GtkStyle * style,
 			       GtkRcStyle * rc_style)
 {
 	ClearlooksStyle *clearlooks_style = CLEARLOOKS_STYLE (style);
-	double shades[] = {1.15, 0.95, 0.896, 0.85, 0.7, 0.665, 0.5, 0.45, 0.4};
+	double shades[] = {1.15, 0.95, 0.896, 0.82, 0.7, 0.665, 0.5, 0.45, 0.4};
 	CairoColor spot_color;
 	CairoColor bg_normal;
 	double contrast;
@@ -1279,19 +1304,18 @@ draw_focus (GtkStyle *style, GdkWindow *window, GtkStateType state_type,
 static void
 clearlooks_style_unrealize (GtkStyle * style)
 {
-	parent_class->unrealize (style);
-
 #ifdef HAVE_ANIMATION
 	 while (signaled_widgets)
 	{
 		if(GTK_IS_CHECK_BUTTON (signaled_widgets->data))
-			g_object_disconnect (signaled_widgets->data, "any_signal::toggled", G_CALLBACK (cl_checkbox_toggle), signaled_widgets->data, NULL);
-		//else if   add other signals here
+			g_object_disconnect (signaled_widgets->data,
+			                     "any_signal::toggled",
+			                     G_CALLBACK (cl_checkbox_toggle),
+			                     signaled_widgets->data, NULL);
+
 		cl_async_animation_remove(signaled_widgets->data);
-		//printf("removed from gslist %d\n",signaled_widgets->data);
 		signaled_widgets = g_slist_next (signaled_widgets);
 	}
-	g_slist_free(signaled_widgets);
 
 	if(async_widgets != NULL)
 	{
@@ -1305,6 +1329,144 @@ clearlooks_style_unrealize (GtkStyle * style)
 		async_widget_timer_id = 0;
 	}
 #endif /* HAVE_ANIMATION */	
+
+	parent_class->unrealize (style);
+}
+
+static GdkPixbuf *
+set_transparency (const GdkPixbuf *pixbuf, gdouble alpha_percent)
+{
+	GdkPixbuf *target;
+	guchar *data, *current;
+	guint x, y, rowstride, height, width;
+
+	g_return_val_if_fail (pixbuf != NULL, NULL);
+	g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
+
+	/* Returns a copy of pixbuf with it's non-completely-transparent pixels to
+	   have an alpha level "alpha_percent" of their original value. */
+
+	target = gdk_pixbuf_add_alpha (pixbuf, FALSE, 0, 0, 0);
+
+	if (alpha_percent == 1.0)
+		return target;
+	width = gdk_pixbuf_get_width (target);
+	height = gdk_pixbuf_get_height (target);
+	rowstride = gdk_pixbuf_get_rowstride (target);
+	data = gdk_pixbuf_get_pixels (target);
+
+	for (y = 0; y < height; y++) {
+		for (x = 0; x < width; x++) {
+			/* The "4" is the number of chars per pixel, in this case, RGBA,
+			   the 3 means "skip to the alpha" */
+			current = data + (y * rowstride) + (x * 4) + 3; 
+			*(current) = (guchar) (*(current) * alpha_percent);
+		}
+	}
+
+	return target;
+}
+
+static GdkPixbuf*
+scale_or_ref (GdkPixbuf *src,
+              int width,
+              int height)
+{
+	if (width == gdk_pixbuf_get_width (src) &&
+	    height == gdk_pixbuf_get_height (src)) {
+		return g_object_ref (src);
+	} else {
+		return gdk_pixbuf_scale_simple (src,
+                                        width, height,
+                                        GDK_INTERP_BILINEAR);
+	}
+}
+
+static GdkPixbuf *
+render_icon (GtkStyle            *style,
+             const GtkIconSource *source,
+             GtkTextDirection     direction,
+             GtkStateType         state,
+             GtkIconSize          size,
+             GtkWidget           *widget,
+             const char          *detail)
+{
+	int width = 1;
+	int height = 1;
+	GdkPixbuf *scaled;
+	GdkPixbuf *stated;
+	GdkPixbuf *base_pixbuf;
+	GdkScreen *screen;
+	GtkSettings *settings;
+	
+	/* Oddly, style can be NULL in this function, because
+	 * GtkIconSet can be used without a style and if so
+	 * it uses this function.
+	 */
+	
+	base_pixbuf = gtk_icon_source_get_pixbuf (source);
+	
+	g_return_val_if_fail (base_pixbuf != NULL, NULL);
+	
+	if (widget && gtk_widget_has_screen (widget)) {
+		screen = gtk_widget_get_screen (widget);
+		settings = gtk_settings_get_for_screen (screen);
+	} else if (style->colormap) {
+		screen = gdk_colormap_get_screen (style->colormap);
+		settings = gtk_settings_get_for_screen (screen);
+	} else {
+		settings = gtk_settings_get_default ();
+		GTK_NOTE (MULTIHEAD,
+			  g_warning ("Using the default screen for gtk_default_render_icon()"));
+	}
+	
+  
+	if (size != (GtkIconSize) -1 && !gtk_icon_size_lookup_for_settings (settings, size, &width, &height)) {
+		g_warning (G_STRLOC ": invalid icon size '%d'", size);
+		return NULL;
+	}
+
+	/* If the size was wildcarded, and we're allowed to scale, then scale; otherwise,
+	 * leave it alone.
+	 */
+	if (size != (GtkIconSize)-1 && gtk_icon_source_get_size_wildcarded (source))
+		scaled = scale_or_ref (base_pixbuf, width, height);
+	else
+		scaled = g_object_ref (base_pixbuf);
+	
+	/* If the state was wildcarded, then generate a state. */
+	if (gtk_icon_source_get_state_wildcarded (source)) {
+		if (state == GTK_STATE_INSENSITIVE) {
+			stated = set_transparency (scaled, 0.3);
+#if 0
+			stated =
+				gdk_pixbuf_composite_color_simple (scaled,
+								   gdk_pixbuf_get_width (scaled),
+								   gdk_pixbuf_get_height (scaled),
+								   GDK_INTERP_BILINEAR, 128,
+								   gdk_pixbuf_get_width (scaled),
+								   style->bg[state].pixel,
+								   style->bg[state].pixel);
+#endif
+			gdk_pixbuf_saturate_and_pixelate (stated, stated,
+							  0.1, FALSE);
+			
+			g_object_unref (scaled);
+		} else if (state == GTK_STATE_PRELIGHT) {
+			stated = gdk_pixbuf_copy (scaled);      
+			
+			gdk_pixbuf_saturate_and_pixelate (scaled, stated,
+							  1.2, FALSE);
+			
+			g_object_unref (scaled);
+		} else {
+			stated = scaled;
+		}
+	}
+	else
+		stated = scaled;
+  
+  return stated;
 }
 
 static void
@@ -1337,6 +1499,7 @@ clearlooks_style_class_init (ClearlooksStyleClass * klass)
 	style_class->draw_hline       = draw_hline;
 	style_class->draw_resize_grip = draw_resize_grip;
 	style_class->draw_arrow       = draw_arrow;
+	style_class->render_icon      = render_icon;
 }
 
 GType clearlooks_type_style = 0;
